@@ -1,7 +1,10 @@
+// Similar to the strings(1) utility
+// We print any sequences involving four or more ASCII letters
 extern crate lua_patterns;
 use lua_patterns::LuaPattern;
 
 use std::env;
+use std::str;
 use std::fs::File;
 use std::io::prelude::*;
 
@@ -11,9 +14,9 @@ fn main() {
     let mut buf = Vec::new();
     f.read_to_end(&mut buf).expect("can't read file");
 
-    let mut words = LuaPattern::new("%a%a+");
+    let mut words = LuaPattern::new("%a%a%a%a+");
     for w in words.gmatch_bytes(&buf) {
-        println!("{}",std::str::from_utf8(w).unwrap());
+        println!("{}",str::from_utf8(w).unwrap());
     }
 
 }
