@@ -10,6 +10,8 @@
 //! [the README](https://github.com/stevedonovan/lua-patterns/blob/master/readme.md)
 //! for more discussion.
 //!
+//! [LuaPattern](struct.LuaPattern.html) implements the public API.
+//!
 //! ## Examples
 //!
 //! ```rust
@@ -39,46 +41,42 @@
 //! }
 //! ```
 
-use std::ptr;
 use std::ops;
-use std::os::raw::{c_int,c_char,c_uint};
-use std::ffi::CStr;
 
-#[repr(C)]
-struct LuaMatch {
-    start: c_int,
-    end: c_int
-}
+pub mod errors;
+use errors::*;
+mod luapat;
+use luapat::*;
 
-static LUA_MAXCAPTURES: usize = 32;
-
-#[link(name = "lua-str", kind="static")]
-extern {
-    fn str_match (
-        s: *const u8, ls: c_uint, p: *const u8, lp: c_uint,
-        err_msg: *mut *mut c_char,
-        mm: *mut LuaMatch
-        ) -> c_int;
-}
 
 /// Represents a Lua string pattern and the results of a match
 pub struct LuaPattern<'a> {
     patt: &'a [u8],
-    matches: Vec<LuaMatch>,
+    matches: [LuaMatch; LUA_MAXCAPTURES],
     n_match: usize
 }
 
 impl <'a> LuaPattern<'a> {
-    /// Create a new Lua pattern from a string
-    pub fn new(patt: &'a str) -> LuaPattern<'a> {
-        LuaPattern::from_bytes(patt.as_bytes())
+    /// Maybe create a new Lua pattern from a slice of bytes
+    pub fn from_bytes_try (bytes: &'a [u8]) -> Result<LuaPattern<'a>,PatternError> {
+        str_check(bytes)?;
+        let matches = [LuaMatch{start: 0, end: 0}; LUA_MAXCAPTURES];
+        Ok(LuaPattern{patt: bytes, matches: matches, n_match: 0})
     }
 
-    /// Create a new Lua pattern from a slice of bytes
+    /// Maybe create a new Lua pattern from a string
+    pub fn new_try(patt: &'a str) -> Result<LuaPattern<'a>,PatternError> {
+        LuaPattern::from_bytes_try(patt.as_bytes())
+    }
+
+    /// Create a new Lua pattern from a string, panicking if bad
+    pub fn new(patt: &'a str) -> LuaPattern<'a> {
+        LuaPattern::new_try(patt).expect("bad pattern")
+    }
+
+    /// Create a new Lua pattern from a slice of bytes, panicking if bad
     pub fn from_bytes (bytes: &'a [u8]) -> LuaPattern<'a> {
-        let mut matches: Vec<LuaMatch> = Vec::with_capacity(LUA_MAXCAPTURES);
-        unsafe {matches.set_len(LUA_MAXCAPTURES);}
-        LuaPattern{patt: bytes, matches: matches, n_match: 0}
+        LuaPattern::from_bytes_try(bytes).expect("bad pattern")
     }
 
     /// Match a slice of bytes with a pattern
@@ -91,20 +89,7 @@ impl <'a> LuaPattern<'a> {
     /// assert_eq!(&bytes[m.range()], &[0xFE,0xEE,0xEE,0xED]);
     /// ```
     pub fn matches_bytes(&mut self, s: &[u8]) -> bool {
-        let c_ptr: *mut c_char = ptr::null_mut();
-        let pvoid = Box::into_raw(Box::new(c_ptr));
-        let err_msg : *mut *mut c_char = pvoid;
-
-        unsafe {
-           self.n_match = str_match(s.as_ptr(),s.len() as c_uint,
-                self.patt.as_ptr(),self.patt.len() as c_uint,
-                err_msg, self.matches.as_mut_ptr()) as usize;
-            let ep = *err_msg;
-            if ! ep.is_null() {
-                panic!(format!("lua-pattern {:?}",CStr::from_ptr(ep)));
-            }
-        }
-
+        self.n_match = str_match(s,self.patt,&mut self.matches).expect("Should not fail - report as bug");
         self.n_match > 0
     }
 
@@ -134,6 +119,43 @@ impl <'a> LuaPattern<'a> {
         }
     }
 
+    /// Match a string, returning first two explicit captures if successful
+    ///
+    /// ```
+    /// let mut p = lua_patterns::LuaPattern::new("%s*(%d+)%s+(%S+)");
+    /// let (int,rest) = p.match_maybe_2(" 233   hello dolly").unwrap();
+    /// assert_eq!(int,"233");
+    /// assert_eq!(rest,"hello");
+    /// ```
+    pub fn match_maybe_2<'t>(&mut self, text: &'t str) -> Option<(&'t str,&'t str)> {
+        if self.matches(text) {
+            let cc = self.match_captures(text);
+            if cc.num_matches() != 3 { return None; }
+            Some((cc.get(1),cc.get(2)))
+        } else {
+            None
+        }
+    }
+
+    /// Match a string, returning first three explicit captures if successful
+    ///
+    /// ```
+    /// let mut p = lua_patterns::LuaPattern::new("(%d+)/(%d+)/(%d+)");
+    /// let (y,m,d) = p.match_maybe_3("2017/11/10").unwrap();
+    /// assert_eq!(y,"2017");
+    /// assert_eq!(m,"11");
+    /// assert_eq!(d,"10");
+    /// ```
+    pub fn match_maybe_3<'t>(&mut self, text: &'t str) -> Option<(&'t str,&'t str,&'t str)> {
+        if self.matches(text) {
+            let cc = self.match_captures(text);
+            if cc.num_matches() != 4 { return None; }
+            Some((cc.get(1),cc.get(2),cc.get(3)))
+        } else {
+            None
+        }
+    }
+
     /// Match and collect all captures as a vector of string slices
     ///
     /// ```
@@ -157,7 +179,7 @@ impl <'a> LuaPattern<'a> {
     ///     assert_eq!(cc.get(1), "hello");
     /// }
     /// ```
-    pub fn match_captures<'b,'c>(&'c mut self, text: &'b str) -> Captures<'a,'b,'c> {
+    pub fn match_captures<'b,'c>(&'c self, text: &'b str) -> Captures<'a,'b,'c> {
         Captures {m: self, text: text}
     }
 
@@ -344,7 +366,7 @@ impl <'a> LuaPattern<'a> {
 }
 
 #[derive(Debug)]
-enum Subst {
+pub enum Subst {
     Text(String),
     Capture(usize)
 }
@@ -355,7 +377,7 @@ impl Subst {
     }
 }
 
-fn generate_gsub_patterns(repl: &str) -> Vec<Subst> {
+pub fn generate_gsub_patterns(repl: &str) -> Vec<Subst> {
     let mut m = LuaPattern::new("%%([%%%d])");
     let mut res = Vec::new();
     let mut slice = repl;
@@ -378,6 +400,32 @@ fn generate_gsub_patterns(repl: &str) -> Vec<Subst> {
     res
 }
 
+pub struct Substitute {
+    repl: Vec<Subst>
+}
+
+impl Substitute {
+    pub fn new(repl: &str) -> Substitute {
+        Substitute{
+            repl: generate_gsub_patterns(repl)
+        }
+    }
+
+    pub fn subst(&self, patt: &LuaPattern, text: &str) -> String {
+        let mut res = String::new();
+        let captures = patt.match_captures(text);
+        for r in &self.repl {
+            match *r {
+                Subst::Text(ref s) => res.push_str(&s),
+                Subst::Capture(i) => res.push_str(captures.get(i))
+            }
+        }
+        res
+    }
+
+}
+
+
 
 /// Low-overhead convenient access to string match captures
 // note: there are three borrows going on here.
@@ -666,6 +714,14 @@ mod tests {
 
     }
 
+    #[test]
+    fn multiple_captures() {
+        let mut p = LuaPattern::new("%s*(%d+)%s+(%S+)");
+        let (int,rest) = p.match_maybe_2(" 233   hello dolly").unwrap();
+        assert_eq!(int,"233");
+        assert_eq!(rest,"hello");
+    }
+
     #[test]
     fn gmatch() {
         let mut m = LuaPattern::new("%a+");
@@ -687,8 +743,6 @@ mod tests {
         assert_eq!(iter.next().unwrap().get(1), "one");
         assert_eq!(iter.next().unwrap().get(1), "two");
         assert_eq!(iter.next().unwrap().get(1), "three");
-
-
     }
 
     #[test]
@@ -719,8 +773,25 @@ mod tests {
         let mut m = LuaPattern::new("(%S+)%s*=%s*(%S+);%s*");
         let res = m.gsub("a=2; b=3; c = 4;", "'%2':%1 ");
         assert_eq!(res,"'2':a '3':b '4':c ");
+    }
 
-
-
+    #[test]
+    fn bad_patterns() {
+       let bad = [
+        ("bonzo %","malformed pattern (ends with '%')"),
+        ("bonzo (dog%(","unfinished capture"),
+        ("alles [%a%[","malformed pattern (missing ']')"),
+        ("bonzo (dog (cat)","unfinished capture"),
+        ("frodo %f[%A","malformed pattern (missing ']')"),
+        ("frodo (1) (2(3)%2)%1","invalid capture index %2"),
+        ];
+        for p in bad.iter() {
+            let res = LuaPattern::new_try(p.0);
+            if let Err(e) = res {
+                assert_eq!(e, PatternError(p.1.into()));
+            } else {
+                panic!("false positive");
+            }
+        }
     }
 }
